#!/usr/bin/perl
#
# cddblookup.pl {username} {sessionnumber}
#
# attempts to look up a CD at a CDDB server to get the album and
# track names.
# uses tags from the session:
#    CDDB                the cddb hash
#    NTRACKS             the number of tracks
#    OFFSETS             list of the track frame offsets
#    TOTLENGTH           total length of disc in frames
#
# if successful, adds tags to the session:
#    CDDB                the cddb hash, now possibly a list
#    ALBUMNAME           the album name
#    ARTISTNAME          the artist name, if it appears in the disc title
#    TRACKNAMEx..        title of each track (including artist - for now)
#    MESSAGE             message indicating success
#    STATE               IDLE
# and it takes a guess at these tags:
#    ARTISTSORT          sort key for artist name
#    ARTISTDIRECTORY     directory name to use for artist
#    ALBUMDIRECTORY      directory name to use for album
#    NAME                the session name, set to album name
#
# the search may instead result in a list of possible matches.
# in this case, the following tags are created:
#    CDDBMATCHES         the number of possible matches
#    CDDBIDx             CDDB ident for each match
#    CDDBDESCx           description for each match
#    STATE               CDDBCHOICE
#
# if unsuccessful,
#    MESSAGE             message indicating the problem
#    STATE               IDLE
#

use FileHandle;
use Socket;

$tunebot_root=$ENV{TUNEBOT_ROOT};
$ses_directory=$tunebot_root."/data/loader";

if ($#ARGV != 1) {
    exit 1;
}
$usrnam=$ARGV[0];
$sesno=$ARGV[1];

#
# parameters
#
$defcddbport=888;
@cddbservers=("cddb.moonsoft.com");
@cddbservers=("sunsite.unc.edu");
@cddbservers=("cddb.celestial.com");
#@cddbservers=("localhost");

$hostname="crashpad";

sub addSesTag {
    open (O, ">>$ses_directory/$usrnam.$sesno");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno";
}

sub getSesTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

sub suggestdir {
    my ($a) = @_;
    $a =~ tr/A-Z/a-z/;
    $a =~ tr/a-z0-9_/_/c;
    $a =~ s/__*/_/g;
    return $a;
}

sub guessSort {
    my ($a) = @_;
    return $a;
}

#
# go fetch the CDDB database entry for a CD
# returns @cddbfile containing the lines of the database entry.
# returns null if succeeded, else error message.
#
sub getcddbfile {
    my ($cddbid, $remote, $port, $iaddr, $paddr, $proto, $line);
    my (@a, $categ, $i);

    $cddbid = getSesTag("CDDB");
    $cddbid =~ s/,.*//;

#    # see if we have the file cached locally
#    $i = glob "/usr/lib/X11/xmcd/cddb/*/" . $albumtags{"CDDB"};
#    if ( -r $i ) {
#	open(I, $i);
#	while ($line = <I>) {
#	    chop $line;
#	    @cddbfile=(@cddbfile, $line);
#	}
#	close(I);
#	return;
#    }

    # open connection to CDDB server
    $remote = $cddbservers[0];
    $port = $defcddbport;
#    print "trying cddb server $remote : $port\n";
    $iaddr = inet_aton($remote) || return "Can't resolve host $remote";
    $paddr = sockaddr_in($port, $iaddr);
    $proto = getprotobyname('tcp');
    socket(SOCK, PF_INET, SOCK_STREAM, $proto) || return "Socket: $!";
    connect(SOCK, $paddr) || return "Connect: $!";
    SOCK->autoflush(1);

    # get hello message
    if (!defined($line = <SOCK>)) {
	close(SOCK);
	return "Protocol error talking to CDDB server";
    }
#    print $line;
    if ($line !~ /^20/) {
	close(SOCK);
	return $line;
    }

    # send hello, get welcome
    print SOCK "cddb hello $usrnam $hostname tunebot v1.0pl0\n";
    if (!defined($line = <SOCK>)) {
	close(SOCK);
	return "Protocol error getting hello";
    }
#    print $line;
    if ($line !~ /^20/) {
	close(SOCK);
	return $line;
    }

    # send query command to get category
    print SOCK "cddb query $cddbid " . getSesTag("NTRACKS");
    foreach $i (split /,/, getSesTag("OFFSETS")) {
	print SOCK " $i";
    }
    print SOCK " " . getSesTag("TOTLENGTH") . "\n";
    if (!defined($line = <SOCK>)) {
	close(SOCK);
	return "Protocol error getting query response";
    }
#    print $line;
    if ($line !~ /^20/) {
	close(SOCK);
	return $line;
    }
    chop $line;
    @a=split(" ", $line);
    $categ=$a[1];
#    print "cddb read $categ $cddbid\n";

    # send read command to get database entry
    print SOCK "cddb read $categ $cddbid\n";
    if (!defined($line = <SOCK>)) {
	close(SOCK);
	return "Protocol error getting read response";
    }
#    print $line;
    if ($line !~ /^21/) {
	close(SOCK);
	return $line;
    }

    # read up database file until '.'
    @cddbfile=();
    while (1) {
	if (!defined($line = <SOCK>)) {
	    close(SOCK);
	    return "Protocol error reading database file";
	}
	if ($line =~ /^\./) {
	    close(SOCK);
	    return;
	}
	chop $line;
	$line =~ s/\r//;
	@cddbfile=(@cddbfile, $line);
    }

#    while (defined($line = <SOCK>)) {
#       print $line;
#    }
#    close(SOCK);
#    return;
}

#
# attempt to fetch the CDDB entry for the CD from a server.
#
# if this works, use it to initialize our tags data, else
# have to start out with nothing.
# there are lots of icky heuristics about album and track names in here.
#
#print "Fetching CDDB entry...\n";
if ($err=getcddbfile()) {
#    print "Error: $err\n";
    addSesTag("MESSAGE=Error looking up CDDB: $err");
    addSesTag("STATE=IDLE");
} else {
    if (0) {
	open(CDDB, ">cddb");
	foreach (@cddbfile) {
	    print CDDB $_, "\n";
	}
	close(CDDB);
    }
    #
    # grind the cddb entry into attributes by concatting lines
    #
    foreach (@cddbfile) {
	$i=index($_, "=");
	if ($i > 0) {
	    $cddb{substr($_, 0, $i)} .= substr($_, $i+1);
	}
    }
#    foreach $i (keys cddb) {
#	print "cddb{$i}=" . $cddb{$i} . "\n";
#    }
    #
    # add any other cddbids to ours
    #
    if ($cddb{"DISCID"} ne "") {
	addSesTag("CDDB=".$cddb{"DISCID"});
    }
    #
    # extract any tags we can from the CDDB entry
    # album name
    # artist
    # track names
    #
    $albumname = $cddb{"DTITLE"};
    $artistname = "";
    if ($albumname =~ /\//) {
	$artistname = $albumname;
	$artistname =~ s/ *\/.*//;
	$albumname =~ s/^[^\/]*\/ *//;
    }
    addSesTag("ARTISTNAME=$artistname");
    addSesTag("ARTISTSORT=".guessSort($artistname));
    addSesTag("ALBUMNAME=$albumname");
    if (getSesTag("NAME") eq "unnamed") {
	addSesTag("NAME=$albumname");
    }
    #
    # track names
    #
    $nslash=0;
    $ntracks = getSesTag("NTRACKS");
    for ($i = 1; $i <= $ntracks; ++$i) {
	if ($t = $cddb{sprintf "TTITLE%d", $i-1}) {
	    addSesTag("TRACKNAME$i=$t");
	    if ($t =~ /  *\/  */) {
		++$nslash;
	    }
	    if ($t =~ /  *-  */) {
		++$ndash;
	    }
	}
    }

    addSesTag("ARTISTDIRECTORY=".suggestdir($artistname));
    addSesTag("ALBUMDIRECTORY=".suggestdir($albumname));
    addSesTag("MESSAGE=Got disc info from CDDB");
    addSesTag("STATE=IDLE");
}

exit 0

